@@ -1,6 +1,7 @@
 from . import by as _by
 from .parameters import parameters
 import os
+import numpy as np
 
 
 class Tier(object):
@@ -19,7 +20,7 @@ class Tier(object):
         self.tier_type = None
 
     def add_point(self, point):
-        """Add point to the Tier."""
+        """Add point (Point object) to the Tier.\nUsage:\n\t(1) You can either (a) create a Point object with \'Point()\', or (b) get a specific Point object from already existing Tiers with \'get_point()\'.\n\t (2) Pass it as the argument."""
         point = Point(point.time, point.value, superior_PitchTier=self)
 
         if point.time < self.start_time or point.time > self.end_time:
@@ -44,7 +45,7 @@ class Tier(object):
         self._indexed = True
 
     def add_points(self, points):
-        """Add points to the Tier."""
+        """Add a list (array-like) of points (Point object) to the Tier.\nUsage:\n\t(1) You can either (a) create the Point objects with \'Point()\', or (b) get specific Point objects from already existing Tiers with \'get_points()\'.\n\t (2) Pass it as the argument."""
 
         for point in points:
             point = Point(point.time, point.value, superior_PitchTier=self)
@@ -72,13 +73,16 @@ class Tier(object):
         self._indexed = True
 
     def to_plot(self, start_time=None, end_time=None):
-        """Plot the points of the Tier from between two time points. If the time points are not specified, start time and end time are used. To change related parameters, use parameters."""
+        """Plot the points of the Tier between two time points. If the time points are not specified, \'start time\' and\\or \'end time\' are used. To change related parameters, use \'parameters\'."""
 
         if start_time == None:
             start_time = self.start_time
         if end_time == None:
             end_time = self.end_time
 
+        if start_time >= end_time:
+            raise Exception('start_time is larger than or equals to end_time.')
+
         df = self.to_dataframe()
 
         this_df = df[df.time >= start_time]
@@ -102,18 +106,18 @@ class Tier(object):
         return plot
 
     def get_point(self, by_target, by):
-        """Get Point of the Tier with chosen method. To change related parameters, use parameters."""
+        """Get Point of the Tier with chosen method. See \'help(by)\' for more information on the available methods. To change related parameters, use \'parameters\'."""
 
-        if by.by_method not in _by.by_methods:
+        if by not in _by.by_methods:
             raise Exception(
                 f'Invalid by_method. Use help() to see valid methods.')
 
-        if by.by_method == 'by_time':
+        if by.by_method == 0:
             for point in self.__points:
                 if point.time == by_target:
                     return point
 
-        elif by.by_method == 'by_index':
+        elif by.by_method == 1:
             try:
                 point = self.__points[by_target-1]
 
@@ -121,7 +125,7 @@ class Tier(object):
             except IndexError:
                 pass
 
-        elif by.by_method == 'by_point':
+        elif by.by_method == 2:
             for point in self.__points:
                 if point == by_target:
                     return point
@@ -130,6 +134,7 @@ class Tier(object):
             raise Exception('Point not found.')
 
     def get_points(self, by_target, by):
+        """Get a list of Point objects with chosen method. See \'help(by)\' for more information on the available methods. To change related parameters, use \'parameters\'."""
 
         points = []
 
@@ -137,7 +142,7 @@ class Tier(object):
             raise Exception(
                 f'Invalid by_method. Use help() to see valid methods.')
 
-        if by.by_method == 'by_point':
+        if by.by_method == 2:
             for point in by_target:
                 if point in self.__points:
                     points.append(point)
@@ -146,11 +151,11 @@ class Tier(object):
                 else:
                     raise Exception('Point not found.')
 
-        elif by.by_method == 'by_index':
+        elif by.by_method == 1:
             for index in range(by_target[0], by_target[1]+1):
-                points.append(self.get_point(index, _by.Index()))
+                points.append(self.get_point(index, _by.INDEX))
 
-        elif by.by_method == 'by_time':
+        elif by.by_method == 0:
             for point in self.__points:
                 if by_target[0] <= point.time <= by_target[1]:
                     points.append(point)
@@ -158,7 +163,7 @@ class Tier(object):
         return points
 
     def remove_point(self, by_target, by):
-        """Remove Point of the Tier with chosen method. To change related parameters, use parameters."""
+        """Remove a Point object of the Tier with chosen method. See \'help(by)\' for more information on the available methods. To change related parameters, use \'parameters\'."""
 
         if by.by_method not in _by.by_methods:
             raise Exception(
@@ -176,7 +181,7 @@ class Tier(object):
         self._indexed = True
 
     def remove_points(self, by_target, by):
-        """Remove Point of the Tier with chosen method. To change related parameters, use parameters."""
+        """Remove Point objects of the Tier with chosen method. See \'help(by)\' for more information on the available methods. To change related parameters, use \'parameters\'."""
 
         if by.by_method not in _by.by_methods:
             raise Exception(
@@ -195,7 +200,7 @@ class Tier(object):
         self._indexed = True
 
     def shift_point_to_time(self, target_time, by_target, by):
-        """Shift Point of the Tier to chosen time with chosen method. To change related parameters, use parameters."""
+        """Shift Point of the Tier to chosen time with chosen method. To change related parameters, use \'parameters\'."""
 
         if by.by_method not in _by.by_methods:
             raise Exception(
@@ -215,25 +220,13 @@ class Tier(object):
             self.add_point(point)
 
     def batch_raise_value(self, by_target, by, value):
-        """Batch raise or lower value of points of the Tier. To change related parameters, use parameters."""
+        """Batch raise or lower value of points of the Tier. To change related parameters, use \'parameters\'."""
 
         points = self.get_points(by_target, by)
 
         for point in points:
             point.value += value
 
-    # def mean(self, start_time=None, end_time=None):
-    #     """Return the mean of the values of the points of the Tier from between two time points. If the time points are not specified, start time and end time are used."""
-    #     if start_time==None:
-    #         start_time=self.start_time
-
-    #     if end_time==None:
-    #         end_time=self.end_time
-
-    #     values = [value for value in self.__points]
-
-    #     return sum(values)/len(values)
-
     def to_dataframe(self):
         """Convert the Tier to dataframe."""
         import pandas as pd
@@ -257,7 +250,7 @@ class Tier(object):
         return df
 
     def write_to_file(self, file=''):
-        """Write the Tier to file. By default saves to the original directory, if any. To change related parameters, use parameters."""
+        """Write the Tier to file. By default saves to the original directory, if any. To change related parameters, use \'parameters\'."""
 
         if file == '':
             file = f'{self.original_dir}/{self.name}.{self.tier_type}'
@@ -330,7 +323,7 @@ class Tier(object):
 
     @start_time.setter
     def start_time(self, start_time):
-        if start_time < 0:
+        if start_time < 0 or np.isnan(start_time):
             raise Exception('Invalid start_time.')
         self.__start_time = start_time
 
@@ -342,7 +335,7 @@ class Tier(object):
 
     @end_time.setter
     def end_time(self, end_time):
-        if end_time < 0:
+        if end_time < 0 or np.isnan(end_time):
             raise Exception('Invalid end_time.')
         self.__end_time = end_time
 
@@ -389,9 +382,9 @@ class Point(object):
 
     def __init__(self, time, value, point_index=None, superior_PitchTier=None):
 
-        if value < 0:
+        if value < 0 or np.isnan(value):
             raise Exception('Invalid value.')
-        if time < 0:
+        if time < 0 or np.isnan(time):
             raise Exception('Invalid time.')
         if point_index != None:
             if point_index < 1:
@@ -411,7 +404,7 @@ class Point(object):
     @time.setter
     def time(self, time):
 
-        if time < 0:
+        if time < 0 or np.isnan(time):
             raise Exception('Invalid time.')
         self.__time = time
 
@@ -427,7 +420,7 @@ class Point(object):
     @value.setter
     def value(self, value):
 
-        if value < 0:
+        if value < 0 or np.isnan(value):
             raise Exception('Invalid pitch value.')
         self.__value = value
 
